--------------------------------------------------------------
--                                                          --
-- BusyObjects SCASM import utility for gmax, v0.1          --
-- Copyright (c) 2002 by Bernd R. Fix <brf@brainon.ch>      --
--                                                          --
-- Based on code by (c) Copyright Wayland Reid, 2002        --
-- Based on code by (c) Copyright Zaddix, 2002              --
--                                                          --
-- Use this code in any way you see fit.                    --
-- If you use any/all of the code in your own scripts,      --
-- just credit all authors somewhere.                       --
--                                                          --
--------------------------------------------------------------

---------------------
-- Tokenizer class --
---------------------

struct Tokenizer
(
	-- local variables
	tokens = #(),
	hdl,
	lineNum,

	-- set file handle
	fn SetStream stream =
	(
		hdl = stream
		lineNum = 0
	),
			  
	-- get next line; returns false on end-of-file
	fn NextLine =
	(
		local line = ""
		while ((line == "") and not (eof hdl)) do
		(
			line = trimright (readDelimitedString hdl "\n")
			lineNum += 1
		)
		line = trimleft(line)
		tokens = filterString line "\t "
		not (eof hdl)
	),

	-- get current line number
	fn GetLineNum =
	(
		lineNum
	),
	               
	-- read next string token
	fn ReadToken =
	(
		if tokens.count > 0 then
		(
			local tok = tokens[1]
			deleteItem tokens 1
			tok
		)
		else
		(
			undefined
		)
	),
               
	-- read next integer token
	fn ReadInteger =
	(
		(ReadToken() as integer)
	),

	-- read next point token
	fn ReadPoint =
	(
		local x  = ReadInteger();
		local z  = ReadInteger();
		local y  = ReadInteger();
		(point3 x y z)
	)
)

-------------------------
-- ImportFile function --
-------------------------
		 
fn ImportSCA pFile=
(             
	local currentPath = getFilenamePath pFile

	-- Open up the file
	local inStream = openFile pFile mode:"r"

	try (
		-- initialize tokenizer
		local tok = Tokenizer()
		tok.SetStream (inStream) 
        local shape = undefined

		-- Go through the whole file
		while tok.NextLine() do
		(
			-- Read the tag
			local idToken = tok.ReadToken()
			-- format "token is '%'\n" idToken

			-- starting a new road?
			local pos
			if (idToken == "RRStart(") then
			(
				if (shape != undefined) then
				(
					format "Finishing curve...\n"
					updateShape shape
				)

				-- create a new line object
				shape = splineShape pos:[0, 0, 0]
				addNewSpline shape
				format "Starting a new curve...\n"

				local r1 = tok.readInteger()
				local r2 = tok.readInteger()
				pos = tok.ReadPoint()
			)
			-- continuing a road definition
			else if (idToken == "RoadCont(") then
			(
				pos = tok.ReadPoint()
			)
			-- unknown content; skip further processing
			else
				continue

			-- add vertex to current spline
			format "Vertex added...\n"
			addKnot shape 1 #smooth #curve pos
		)
		-- closing a unfinished shape
		if (shape != undefined) then
		(
			format "Finishing curve...\n"
			updateShape shape
		)
	)
	catch
	(
		messageBox ("BusyObjects import error at line:" + tok.GetLineNum() as string) 
		close inStream
		throw
	)
	close inStream
)


-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
---
---   Rollout definitions
---
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------

rollout ImportRollout "Import SCA" width:160 height:192
(
	groupBox igrp4 "Filename" pos:[8,8] width:144 height:72
	editText import_filename_edit "" pos:[16,24] width:128 height:16
	button import_browse_button "Browse" pos:[72,48] width:70 height:24
	button import_button "Import" pos:[32,152] width:104 height:24

	on import_browse_button pressed do 
	(
		local filename = getOpenFileName caption:"Import mdl" types:"SCASM source (*.sca)|*.sca|All Files (*.*)|*.*|"
		if (filename == undefined) then filename = ""
		import_filename_edit.text = filename
	)

	on import_button pressed do
	(
		local fName = import_filename_edit.text
  
		clearListener()
		format "Importing %\r\n" fName
					  
		ImportSCA fName
		  
		-- Redraw gmax viewports
		clearSelection()
		max tool zoomextents all
		max views redraw

		format "Finished.\n\r\n"
	)
)


-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
---
---   Acutal window creation
---
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------

-- create the rollout window 
if ImportSCA_tool != undefined do
(
	closerolloutfloater ImportSCA_tool
)
		
ImportSCA_tool = newRolloutFloater "SCA Import Tool" 189 480 0 100
addRollout ImportRollout	ImportSCA_tool

if IsSceneRedrawDisabled()==true do enablesceneredraw()

